/* 
   Copyright (c) 1995 by Cisco systems, Inc.
   All rights reserved.

   Please NOTE:  None of the TACACS code available here comes with any
   warranty or support.
*/

#include "tac_plus.h"

#if defined(SYSV) || defined(BSD4_4)

#include <pwd.h>
#include <string.h>

static struct passwd pw_passwd;
static FILE *passwd_fp = NULL;
static char *pw_filename = NULL;

struct passwd *
getpwnam(name)
#if defined(SUNOS5_4) || defined(BSD4_4)
const
#endif
char *name;
{
    static char uname[512];
    static char password[1024];
    static char gecos[1024];
    static char homedir[1024];
    static char shell[1024];
    char buf[1024];
    char *s, *e;

    if (!passwd_fp) {
	if (!pw_filename) {
	    passwd_fp = fopen("/etc/passwd", "r");
	    if (debug & DEBUG_AUTHEN_FLAG)
		report(LOG_DEBUG, 
		       "open %s %d", "/etc/passwd", fileno(passwd_fp));
	} else {
	    passwd_fp = fopen(pw_filename, "r");
	    if (debug & DEBUG_AUTHEN_FLAG)
		report(LOG_DEBUG, 
		       "open %s %d", pw_filename, fileno(passwd_fp));	
	}
    }

    if (passwd_fp == NULL)
	return NULL;

    while (fgets(buf, sizeof(buf), passwd_fp)) {

	/* uname, password, uid, gid, gecos, homedir, shell */

	s = buf;
	e = index(buf, ':');
	if (!e)
	    return(NULL);
	strncpy(uname, s, e - s);
	uname[e - s] = '\0';

	/* try next entry */
	if (strcmp(uname, name))
	    continue;

	s = e + 1;
	e = index(s, ':');
	if (!e)
	    return(NULL);
	strncpy(password, s, e - s);
	password[e - s] = '\0';

	s = e + 1;
	e = index(s, ':');
	if (!e)
	    return(NULL);
	pw_passwd.pw_uid = atoi(s);

	s = e + 1;
	e = index(s, ':');
	pw_passwd.pw_gid = atoi(s);

	s = e + 1;
	e = index(s, ':');
	if (!e)
	    return(NULL);
	strncpy(gecos, s, e - s);
	gecos[e - s] = '\0';

	s = e + 1;
	e = index(s, ':');
	if (!e)
	    return(NULL);
	strncpy(homedir, s, e - s);
	homedir[e - s] = '\0';

	s = e + 1;
	e = index(s, '\n');
	if (!e)
	    return(NULL);
	strncpy(shell, s, e - s);
	shell[e - s] = '\0';

        pw_passwd.pw_name    = uname;
        pw_passwd.pw_passwd  = password;
#ifndef BSD4_4
        pw_passwd.pw_age     = NULL;
        pw_passwd.pw_comment = NULL;
#endif /* BSD4_4 */
        pw_passwd.pw_gecos   = gecos;
        pw_passwd.pw_dir     = homedir;
        pw_passwd.pw_shell   = shell;

	if (debug & DEBUG_AUTHEN_FLAG)
	    report(LOG_DEBUG, "close %d", fileno(passwd_fp));
	fclose(passwd_fp);
	passwd_fp = NULL;
	return(&pw_passwd);
    }

    /* no match found */
    if (debug & DEBUG_AUTHEN_FLAG)
	report(LOG_DEBUG, "close %d", fileno(passwd_fp));
    fclose(passwd_fp);
    passwd_fp = NULL;
    return NULL;
}

void
setpwent()
{
    if (passwd_fp != NULL) { 
	if (debug & DEBUG_AUTHEN_FLAG)
	    report(LOG_DEBUG, "close %d", fileno(passwd_fp));
	fclose(passwd_fp);
    }
    
    if (!pw_filename) {
	passwd_fp = fopen("/etc/passwd", "r");
	if (debug & DEBUG_AUTHEN_FLAG)
	    report(LOG_DEBUG, 
		   "open %s %d", "/etc/passwd", fileno(passwd_fp));
    } else {
	passwd_fp = fopen(pw_filename, "r");
	if (debug & DEBUG_AUTHEN_FLAG)
	    report(LOG_DEBUG, 
		   "open %s %d", pw_filename, fileno(passwd_fp));
    }
}

void
endpwent()
{
    if (passwd_fp != NULL) {
	if (debug & DEBUG_AUTHEN_FLAG)
	    report(LOG_DEBUG, "close %d", fileno(passwd_fp));
	fclose(passwd_fp);
    }
    passwd_fp = NULL;
}

int 
setpwfile(file)
    char *file;
{
    endpwent();
    if (pw_filename)
	free(pw_filename);
    pw_filename = strdup(file);
    if (pw_filename)
	return(1);
    return(0);
}

#endif /* SYSV */
